/*!
 * @author electricessence / https://github.com/electricessence/
 * Licensing: MIT https://github.com/electricessence/TypeScript.NET/blob/master/LICENSE.md
 */
System.register(["../Types"], function (exports_1, context_1) {
    "use strict";
    var __moduleName = context_1 && context_1.id;
    /**
     * Takes any number of disposables as arguments and attempts to dispose them.
     * Any exceptions thrown within a dispose are not trapped.
     * Use 'disposeWithoutException' to automatically trap exceptions.
     *
     * Can accept <any> and will ignore objects that don't have a dispose() method.
     * @param disposables
     */
    function dispose() {
        var disposables = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            disposables[_i] = arguments[_i];
        }
        // The disposables arguments array is effectively localized so it's safe.
        disposeTheseInternal(disposables, false);
    }
    exports_1("dispose", dispose);
    /**
     * Just like in C# this 'using' function will ensure the passed disposable is disposed when the closure has finished.
     *
     * Usage:
     * ```typescript
     * using(new DisposableObject(),(myObj)=>{
         *   // do work with myObj
         * });
     * // myObj automatically has it's dispose method called.
     * ```
     *
     * @param disposable Object to be disposed.
     * @param closure Function call to execute.
     * @returns {TReturn} Returns whatever the closure's return value is.
     */
    function using(disposable, closure) {
        try {
            return closure(disposable);
        }
        finally {
            disposeSingle(disposable, false);
        }
    }
    exports_1("using", using);
    /**
     * This private function makes disposing more robust for when there's no type checking.
     * If trapExceptions is 'true' it catches and returns any exception instead of throwing.
     */
    function disposeSingle(disposable, trapExceptions) {
        if (disposable
            && typeof disposable == Types_1.Type.OBJECT
            && typeof disposable['dispose'] == "function") {
            if (trapExceptions) {
                try {
                    disposable.dispose();
                }
                catch (ex) {
                    return ex;
                }
            }
            else
                disposable.dispose();
        }
        return null;
    }
    /**
     * This dispose method assumes it's working on a local arrayCopy and is unsafe for external use.
     */
    function disposeTheseInternal(disposables, trapExceptions, index) {
        if (index === void 0) { index = 0; }
        var exceptions;
        var len = disposables ? disposables.length : 0;
        for (; index < len; index++) {
            var next = disposables[index];
            if (!next)
                continue;
            if (trapExceptions) {
                var ex = disposeSingle(next, true);
                if (ex) {
                    if (!exceptions)
                        exceptions = [];
                    exceptions.push(ex);
                }
            }
            else {
                var success = false;
                try {
                    disposeSingle(next, false);
                    success = true;
                }
                finally {
                    if (!success && index + 1 < len) {
                        /* If code is 'continued' by the debugger,
                         * need to ensure the rest of the disposables are cared for. */
                        disposeTheseInternal(disposables, false, index + 1);
                    }
                }
                // Just in case...  Should never happen, but asserts the intention.
                if (!success)
                    break;
            }
        }
        return exceptions;
    }
    var Types_1;
    return {
        setters: [
            function (Types_1_1) {
                Types_1 = Types_1_1;
            }
        ],
        execute: function () {/*!
             * @author electricessence / https://github.com/electricessence/
             * Licensing: MIT https://github.com/electricessence/TypeScript.NET/blob/master/LICENSE.md
             */
            (function (dispose) {
                /**
                 * Use this when only disposing one object to avoid creation of arrays.
                 * @param disposable
                 * @param trapExceptions
                 */
                function single(disposable, trapExceptions) {
                    if (trapExceptions === void 0) { trapExceptions = false; }
                    if (disposable)
                        disposeSingle(disposable, trapExceptions);
                }
                dispose.single = single;
                function deferred() {
                    var disposables = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        disposables[_i] = arguments[_i];
                    }
                    these.deferred(disposables);
                }
                dispose.deferred = deferred;
                /**
                 * Takes any number of disposables and traps any errors that occur when disposing.
                 * Returns an array of the exceptions thrown.
                 * @param disposables
                 * @returns {any[]} Returns an array of exceptions that occurred, if there are any.
                 */
                function withoutException() {
                    var disposables = [];
                    for (var _i = 0; _i < arguments.length; _i++) {
                        disposables[_i] = arguments[_i];
                    }
                    // The disposables arguments array is effectively localized so it's safe.
                    return disposeTheseInternal(disposables, true);
                }
                dispose.withoutException = withoutException;
                /**
                 * Takes an array of disposable objects and ensures they are disposed.
                 * @param disposables
                 * @param trapExceptions If true, prevents exceptions from being thrown when disposing.
                 * @returns {any[]} If 'trapExceptions' is true, returns an array of exceptions that occurred, if there are any.
                 */
                function these(disposables, trapExceptions) {
                    return disposables && disposables.length
                        ? disposeTheseInternal(disposables.slice(), trapExceptions)
                        : void 0;
                }
                dispose.these = these;
                (function (these) {
                    function deferred(disposables, delay) {
                        if (delay === void 0) { delay = 0; }
                        if (disposables && disposables.length) {
                            if (!(delay >= 0))
                                delay = 0;
                            setTimeout(disposeTheseInternal, delay, disposables.slice(), true);
                        }
                    }
                    these.deferred = deferred;
                    /**
                     * Use this unsafe method when guaranteed not to cause events that will make modifications to the disposables array.
                     * @param disposables
                     * @param trapExceptions
                     * @returns {any[]}
                     */
                    function noCopy(disposables, trapExceptions) {
                        return disposables && disposables.length
                            ? disposeTheseInternal(disposables, trapExceptions)
                            : void 0;
                    }
                    these.noCopy = noCopy;
                })(these = dispose.these || (dispose.these = {}));
            })(dispose || (dispose = {}));
            exports_1("dispose", dispose);
            exports_1("default", dispose);
        }
    };
});
//# sourceMappingURL=dispose.js.map